@@ -17,7 +17,6 @@ mod cmd_test;
 
 use autocxx_engine::parse_file;
 use clap::{crate_authors, crate_version, App, Arg, ArgGroup};
-use indoc::indoc;
 use proc_macro2::TokenStream;
 use quote::ToTokens;
 use std::io::{Read, Write};
@@ -26,7 +25,7 @@ use std::{fs::File, path::Path};
 
 pub(crate) static BLANK: &str = "// Blank autocxx placeholder";
 
-static LONG_HELP: &str = indoc! {"
+static LONG_HELP: &str = "
 Command line utility to expand the Rust 'autocxx' include_cpp! directive.
 
 This tool can generate both the C++ and Rust side binding code for
@@ -70,7 +69,7 @@ a) Have exactly one include_cpp directive in each source .rs file
 b) Set AUTOCXX_RS_FILE when using autocxx_macro.
 c) Teach your build system always that the outputs of this tool
    are always guaranteed to be gen0.include.rs, gen0.cc and gen1.cc.
-"};
+";
 
 fn main() {
     let matches = App::new("autocxx-gen")
@@ -152,6 +151,11 @@ fn main() {
                 .help("Make the name of the .rs file predictable. You must set AUTOCXX_RS_FILE during Rust build time to educate autocxx_macro about your choice.")
                 .requires("gen-rs-include")
         )
+        .arg(
+            Arg::with_name("auto-allowlist")
+                .long("auto-allowlist")
+                .help("Dynamically construct allowlist from real uses of APIs.")
+        )
         .arg(
             Arg::with_name("clang-args")
                 .last(true)
@@ -161,8 +165,11 @@ fn main() {
         .get_matches();
 
     env_logger::builder().init();
-    let mut parsed_file = parse_file(matches.value_of("INPUT").unwrap())
-        .expect("Unable to parse Rust file and interpret autocxx macro");
+    let mut parsed_file = parse_file(
+        matches.value_of("INPUT").unwrap(),
+        matches.is_present("auto-allowlist"),
+    )
+    .expect("Unable to parse Rust file and interpret autocxx macro");
     let incs = matches
         .values_of("inc")
         .unwrap_or_default()
@@ -191,7 +198,7 @@ fn main() {
                 .expect("Unable to generate header and C++ code");
             for pair in generations.0 {
                 let cppname = format!("gen{}.{}", counter, cpp);
-                write_to_file(&outdir, cppname, &pair.implementation);
+                write_to_file(&outdir, cppname, &pair.implementation.unwrap_or(Vec::new()));
                 write_to_file(&outdir, pair.header_name, &pair.header);
                 counter += 1;
             }
